/** readn.c
*
*   Read or write n bytes from/to a socket descriptor.
*   This version strips parity bits.
*
*   W.G.J. Langeveld
*
**/
#include <sys/socket.h>
/*
*   Prototypes
*/
#include "readn.h"

/**
*
*   Read n bytes from a socket descriptor.
*
**/
long  readn(long fd, char *ptr, long nbytes)
{
   long nleft, nread, i;

   nleft = nbytes;
   while (nleft > 0) {
      nread = recv(fd, ptr, nleft, 0);

      if      (nread  < 0) return(nread);
      else if (nread == 0) break;

      for (i = 0; i < nread; i++) ptr[i] &= 0x7F;

      nleft -= nread;
      ptr   += nread;
   }

   return(nbytes - nleft);
}


/**
*
*   Write n bytes to a socket descriptor.
*
**/
long  writen(long fd, char *ptr, long nbytes)
{
   long nleft, nwritten, i;

   for (i = 0; i < nbytes; i++) ptr[i] &= 0x7F;

   nleft = nbytes;
   while (nleft > 0) {
      nwritten = send(fd, ptr, nleft, 0);

      if (nwritten <= 0) return(nwritten);

      nleft -= nwritten;
      ptr   += nwritten;
   }

   return(nbytes - nleft);
}


/**
*
*   Read a linefeed-terminated line from the host. The returned
*   length does not include the linefeed.
*
**/
long readline(long fd, char *ptr, long maxlen)
{
   long n, rc;
   unsigned char c;

   for (n = 0; n < (maxlen - 1); n++) {
      rc = recv(fd, &c, 1, 0);
      if (rc == 1) {
/*
*   Suppose we get a 0. Then we might as well add a linefeed and quit.
*/
         if (c == '\0') c = '\n';

         *ptr++ = c & 0x7F;

         if (c == '\n') {
/*
*   Fix up cases with \r\n
*/
            if (n > 0) {
               if (ptr[-2] == '\r') {
                  ptr  -= 2;
                 *ptr++ = '\n';
                  n--;
               }
            }
/*
*   The break here makes sure that the line feed is not included in the
*   string length count.
*/
            break;
         }
      }
      else {
         if (rc < 0) n = -1;
         break;
      }
   }

   *ptr = 0;

   return(n);
}

